package org.processmining.autoassociationplot;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;

import weka.core.ContingencyTables;

@Plugin(name = "Visualize Log with Auto-association Plot", returnLabels = { "Auto Auto-association Plot" }, returnTypes = { JComponent.class }, parameterLabels = { "Event Log" }, userAccessible = true)
@Visualizer
public class AutoAssociationPlot {

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(UIPluginContext context, XLog log) {
		JPanel pane = new JPanel();
		pane.setLayout(new java.awt.BorderLayout());
		
		Set<String> activities = new HashSet<String>();
		Map<Integer, Table<String, String, Long>> lagToMatrixMap = new HashMap<Integer, Table<String, String, Long>>();
		Map<Integer, Integer> lagToTraceCountMap = new HashMap<Integer, Integer>();
		for(XTrace trace : log){
			for(int i=0; i<trace.size(); i++){
				String activity = trace.get(i).getAttributes().get("concept:name").toString();
				activities.add(activity);
				for(int lag=1; lag<=i; lag++){
					String laggedActivty = trace.get(i-lag).getAttributes().get("concept:name").toString();
					Table<String, String, Long> matrix = lagToMatrixMap.get(lag);
					if(matrix==null)
						matrix = HashBasedTable.create();
					Long currentCount = matrix.get(laggedActivty, activity);
					if(currentCount == null)
						currentCount = 0l;
					currentCount++;
					matrix.put(laggedActivty, activity, currentCount);
					lagToMatrixMap.put(lag, matrix);
				}
				if(i<(trace.size()-1)){
					Integer traceCount = lagToTraceCountMap.get(i+1);
					if(traceCount == null)
						traceCount = 0;
					traceCount++;
					lagToTraceCountMap.put(i+1, traceCount);
				}
			}
		}
		
		List<String> activitiesAsList = new ArrayList<String>(activities);
	    XYSeriesCollection dataset = new XYSeriesCollection();
	    XYSeries trace_ratio = new XYSeries("Trace ratio");
	    XYSeries tau = new XYSeries("Goodman and Kruskal's tau");
	    for(int key : lagToMatrixMap.keySet()){
			Table<String, String, Long> matrix = lagToMatrixMap.get(key);
			double[][] matrixAsMatrix = new double[activitiesAsList.size()][activitiesAsList.size()];
			int i=0;
			for(String activity1 : activitiesAsList){
				int j=0;
				for(String activity2 : activitiesAsList){
					Long l = matrix.get(activity1, activity2);
					if(l==null)
						l=0l;
					matrixAsMatrix[i][j] = l;
					j++;
				}
				i++;
			}
			Integer numOfTraces = lagToTraceCountMap.get(key);
			if(numOfTraces==null)
				numOfTraces = 0;
			double traceRatio = ((double) numOfTraces) / log.size();
			trace_ratio.add(key, traceRatio);
			double tauval = ContingencyTables.tauVal(matrixAsMatrix);
			tau.add(key, tauval);
			//double cramerd = ContingencyTables.CramersV(matrixAsMatrix);
			//dataset.addValue(cramerd, "Cramer's d", new Integer(key));
		}

	    dataset.addSeries(trace_ratio);
	    dataset.addSeries(tau);
	    JFreeChart lineChart = ChartFactory.createXYLineChart("Auto-association plot", "Lag", "Association", dataset, PlotOrientation.VERTICAL, true,true,false);
	    ChartPanel chartPanel = new ChartPanel(lineChart);
	    NumberAxis domain = new NumberAxis("Lag");
	    domain.setRange(1, Collections.max(lagToMatrixMap.keySet()));
	    domain.setTickUnit(new NumberTickUnit(1.0));
	    lineChart.getXYPlot().setDomainAxis(domain);
	    lineChart.getXYPlot().getDomainAxis().setInverted(true);
	    lineChart.getXYPlot().getRangeAxis().setRange(0,1);
		pane.add(chartPanel);
		pane.validate();

		/*
		JTabbedPane tabbedPane = new JTabbedPane();
		for (int index = 0; index < nets.getSize(); index++) {
			visualize(context, nets, index, visualizer, tabbedPane);
			String label = "Net " + (index + 1);
			tabbedPane.add(label, visualizer.visualize(context, nets.getNet(index)));
		}
		*/
		return pane;
	}
}
